import logging

from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.views.decorators.http import require_POST
from telethon.errors import RPCError

from .exceptions import PayloadException, TelegramAuthorizationException
from .models import TelegramAuthorization
from .utils import parse_json_payload, get_telegram_client

logger = logging.getLogger("django-telethon-authorization")


@login_required
@require_POST
def request_code(request):
    """ Accept JSON {phone: +xxxxxxxxxxx} """
    try:
        phone, = parse_json_payload(request.body, "phone")
    except PayloadException as e:
        logger.warning(e)
        return e.to_response()

    auth, _ = TelegramAuthorization.objects.get_or_create(user=request.user, phone=phone)

    try:
        client = get_telegram_client(phone)
    except TelegramAuthorizationException as e:
        return e.to_response()

    if client.is_user_authorized():
        return JsonResponse({"success": False, "message": "You are already authorized"})
    try:
        response = client.send_code_request(phone)
        auth.phone_code_hash = response.phone_code_hash
        auth.save()
        client.disconnect()
        return JsonResponse({"success": True})

    except RPCError as e:
        return JsonResponse(
            {"success": False, "message": "Telegram exception occurred. %s. %s" % (e.code, e.message)})

    except Exception as e:
        logger.warning("TG REQUEST CODE. POST. Error occurred during telegram send code\n%s" % e)
        return JsonResponse({"success": False, "message": "'Error occurred during code sending\n%s'" % e})


@login_required
@require_POST
def submit(request):
    try:
        phone, code = parse_json_payload(request.body, "phone", "code")
    except PayloadException as e:
        logger.warning(e)
        return e.to_response()

    try:
        auth = TelegramAuthorization.objects.get(user=request.user, phone=phone)
    except TelegramAuthorization.DoesNotExist:
        return JsonResponse({"success": False, "message": "Phone '%s' is invalid'" % phone})

    client = get_telegram_client(phone)

    try:
        client.sign_in(auth.phone, code, phone_code_hash=auth.phone_code_hash)
        client.disconnect()

        # do not store code or hash after successful login
        auth.phone_code_hash = None
        auth.save()

    except RPCError as e:
        return JsonResponse(
            {"success": False, "message": "Telegram exception occurred. %s. %s" % (e.code, e.message)})

    except Exception as e:
        logger.warning("TG Login. POST. Error occurred during telegram sign-in\n%s" % e)
        return JsonResponse({"success": False, "message": "'Error occurred during telegram sign-in\n%s'" % e})
    return JsonResponse({"success": True})


@login_required
@require_POST
def logout(request):
    try:
        phone = parse_json_payload(request.body, "phone")
    except PayloadException as e:
        logger.warning(e)
        return e.to_response()

    try:
        telegram_authorization = TelegramAuthorization.objects.get(user=request.user, phone=phone)
    except TelegramAuthorization.DoesNotExist:
        return JsonResponse({"success": False, "message": "Phone '%s' is invalid'" % phone})

    client = get_telegram_client(telegram_authorization.phone)

    if client.log_out():
        return JsonResponse({"success": True})
    else:
        return JsonResponse({"success": False, "message": "Telegram RPC error"})



from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class TelegramAuthConfig(AppConfig):
    name = 'django_telethon_authorization'

    def ready(self):
        if not (
            getattr(settings, "DTA_TG_SESSION_DIR", None) and
            getattr(settings, "DTA_TG_API_ID", None) and
            getattr(settings, "DTA_TG_API_HASH", None)
        ):
            raise ImproperlyConfigured(
                "\nIn order to use django-telethon-authorization you must set settings variables:\n"
                "DTA_TG_SESSION_DIR\n"
                "DTA_TG_API_ID\n"
                "DTA_TG_API_HASH"
            )



from django.contrib.auth import get_user_model
from django.db import models


class TelegramAuthorization(models.Model):
    user = models.OneToOneField(to=get_user_model(), on_delete=models.CASCADE)
    phone = models.CharField(max_length=254)
    code = models.IntegerField(null=True, blank=True)
    phone_code_hash = models.CharField(max_length=254, null=True, blank=True)
    timestamp = models.DateTimeField(auto_now_add=True)



import json
import os

from django.conf import settings
from telethon import TelegramClient

from .models import TelegramAuthorization
from .exceptions import PayloadException, TelegramAuthorizationException


def parse_json_payload(body, *keys):
    try:
        raw_payload = body.decode("utf-8")
    except UnicodeDecodeError as exc:
        raise PayloadException("Cant decode body '%s'\n%s" % (body, exc))

    try:
        payload = json.loads(raw_payload)
    except (ValueError, TypeError) as exc:
        raise PayloadException("Can't load JSON from raw payload '%s'\n%s" % (raw_payload, exc))

    for key in keys:
        if key not in payload:
            raise PayloadException("Key '%s' was not found in payload '%s'" % (key, payload))

    return (payload.get(key) for key in keys)


def get_telegram_client(phone):
    client = TelegramClient(
        os.path.join(settings.DTA_TG_SESSION_DIR, "%s.telegram_session" % phone),
        settings.DTA_TG_API_ID,
        settings.DTA_TG_API_HASH,
        spawn_read_thread=False,
        report_errors=False
    )
    success = client.connect()
    if not success:
        raise TelegramAuthorizationException("Can't connect to telegram servers")

    return client


def is_authorized(request):
    try:
        telegram_authorization = TelegramAuthorization.objects.get(user=request.user)
    except TelegramAuthorization.DoesNotExist:
        return False
    try:
        client = get_telegram_client(telegram_authorization.phone)
    except TelegramAuthorizationException as e:
        return False

    return client.is_user_authorized()
